#!/bin/bash
#
GPUDB_NAME=GPUdb           # it may be Gaia

echo "This script is used to recovery the data set of $GPUDB_NAME from the backup database"
echo "when $GPUDB_NAME is no longer reading the data in the persist data store."
echo "It assumes that the $GPUDB_NAME HA Backup Processor is running and already backed up data set."
echo "If backup not working, nothing can be restored."
echo "This script should be executed in $GPUDB_NAME HA master node by authorized user, like root or gpudb."
#echo "the exit code 1 means something wrong and can't continue.
echo

function confirm_user() {
  if [ -z "$1" ]; then
    # if no prompt message
    return 1
  fi
  echo $1
  read -p "(y/n) ? " -n 1 -r; echo;
  if [[ "a$REPLY" != "ay" ]]; then
    exit 1
  fi
  echo
}

confirm_user "Do you want to continue?"

# The directory of this script.
# SCRIPT_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

GPUDB_DIR=/opt/gpudb       # the directory where gpudb is installed
GPUDB_HA_DIR=$GPUDB_DIR/ha    # the directory where gpudb HA is installed

# check if gpudb HA is installed locally
if [ ! -d $GPUDB_HA_DIR ]; then
  echo "GPUdb HA is not installed. It's required for data recovery."
  exit 1
fi

# check if HA backup processor is running
ps -U gpudb -f | grep BackupProcessor > /dev/null 2>&1
if [ $? -ne 0 ]; then
  echo "$GPUDB_NAME HA BackupProcessor is not running."
  #exit 1
fi

# find gpudb persist data location
GPUDB_HOST=`awk '{print $1}' $GPUDB_DIR/etc/hostsfile`
GPUDB_PERSIST_DIR=`cat /opt/gpudb/etc/gpudb.conf | grep -oP '(?<=^persist_directory =).*'`

# check gpudb persist data version
if [ -f $GPUDB_PERSIST_DIR/version ]; then
  echo "$GPUDB_NAME persist version is `cat $GPUDB_PERSIST_DIR/version`"
fi

# get gpudb version
# echo "$GPUDB_NAME version is `$GPUDB_DIR/bin/gpudb --version`"

# if gpudb is running locally we need to stop it.
status=`service gpudb status`
if [[ "$status" =~ "GPUdb is running" ]]; then
  echo "$GPUDB_NAME is running and it needs to be stopped in order to safely clear persist data."
  confirm_user "Can it be stopped now?"
  echo "Stopping GPUdb ..."
  service gpudb stop
fi

# if gpudb-ha is running locally we need to stop it.
status=`service gpudb-ha status`
if [[ "$status" =~ "HAProc is running" ]]; then
  echo "Stopping GPUdb HA ..."
  service gpudb-ha stop
fi

# clearing gpudb persist data files, first temporarily make a copy in case of any failure in the following steps.
TEMP_DIR=/tmp/gpudb/persist/`date +"%Y%m%d"`
if [ -d $TEMP_DIR ]; then
  rm -rf $TEMP_DIR/*
else
  mkdir -p $TEMP_DIR
fi
cp -rf $GPUDB_PERSIST_DIR/* $TEMP_DIR/
# deleting persis data files
$GPUDB_DIR/bin/gpudb_hosts_persist_clear.sh

# restart gpudb, restore needs it.
echo "$GPUDB_NAME needs to be restarted in order to restore persist data."
confirm_user "Can it be restarted now?"
echo "Restarting GPUdb ..."
service gpudb start

# find gpudb url
GPUDB_HA_CONFIG=/opt/gpudb/ha/conf/ha-proc-conf.xml
names=($(cat $GPUDB_HA_CONFIG | grep -oP '(?<=name>)[^<]+'))
values=($(cat $GPUDB_HA_CONFIG | grep -oP '(?<=value>)[^<]+'))

for i in ${!names[*]}; do
  if [ "${names[$i]}" == "child.store" ]; then
    GPUDB_URL="${values[$i]}"
    echo "GPUDB_URL is $GPUDB_URL"
    break;
  fi
done

# to make sure gpudb is started and running
for i in {1..10}; do
  status=`curl -sL -w "%{http_code}\\n" "$GPUDB_URL" -o /dev/null`
  if [ $status -eq 200 ]; then
    echo "$GPUDB_URL is running"
    break;
  else
    sleep 1s
  fi
done

# restore the data from the backup data store
# assume that the HA backup process already copied all gpudb data to the backup database
confirm_user "Can it start restore process?"
echo "Restoring GPUdb data from the backup database ..."
$GPUDB_HA_DIR/bin/restore-proc

# restart gpudb HA
confirm_user "Do you want to restart $GPUDB_NAME HA process?"
service gpudb-ha start

# DONE THE JOB.
echo "The job is done, please verify if $GPUDB_NAME is working. Please report any errors occurred."
  